#==========================================================================
# OSL calibration method test
# by Yury Kuchura EU1KY kuchura@gmail.com
#==========================================================================

# Simulated measurements are taken from wonderful SimSmith 9p9 Java applet by AE6TY
# Using F = 14.0 MHz, using 1.2m RG-213/U Belden cable between load and impedance analyzer

# Function to calculate Gamma (complex reflection coefficient) from complex impedance Z
# using formula G = (Zl - Zs) / (Zl + Zs), where Zs = 50.0+0j
def Gamma(Z):
    return (Z - 50.0) / (Z + 50.0)

# Function to calculate Z from G using the above formula
def ZFromG(G):
    Z = (50.0 * G + 50.0) / (1.0 - G)
    return Z

#==========================================================================
# OSL calibration measurements. Assuming we properly measured Z = (R + jX)
#==========================================================================

#Short load: 10 Ohm at the cable end
Zl1 = 10.0             #Actual (known) impedance
G1 = Gamma(Zl1)        #Actual (known) load reflection coefficient
Zm1 = 13.684 + 28.212j #Measured impedance
Gm1 = Gamma(Zm1)       #Measured reflection coefficient

#Nominal load: 50 Ohm at the cable end
Zl2 = 50.0              #Actual (known) impedance
G2 = Gamma(Zl2)         #Actual (known) load reflection coefficient
Zm2 = 50.331 + 0.08069j #Measured impedance
Gm2 = Gamma(Zm2)        #Measured reflection coefficient

#Open load: 300 Ohm at the cable end
Zl3 = 300.0             #Actual (known) impedance
G3 = Gamma(Zl3)         #Actual (known) load reflection coefficient
Zm3 = 30.025 - 76.144j  #Measured impedance
Gm3 = Gamma(Zm3)        #Measured reflection coefficient

#==========================================================================

# Now we have the system of 3 equations with three unknown coefficients
# (e00, e11 and de) that can be used to recover actual impedance of unknown load
# at the end of cable. These complex coefficients allow recovering any load
# impedance.
# e00 + (G1 * Gm1) * e11 - G1 * de = Gm1
# e00 + G2 * Gm2 * e11 - G2 * de = Gm2
# e00 + G3 * Gm3 * e11 - G3 * de = Gm3
# We can use Kramer method to solve these equations

# Function to calculate determinant of 3x3 matrix
# Input: 3x3 matrix [[a, b, c], [m, n, k], [u, v, w]]
def Determinant3(mtx):
    a = mtx[0][0]
    b = mtx[0][1]
    c = mtx[0][2]
    m = mtx[1][0]
    n = mtx[1][1]
    k = mtx[1][2]
    u = mtx[2][0]
    v = mtx[2][1]
    w = mtx[2][2]
    return a*n*w + b*k*u + m*v*c - c*n*u - b*m*w - a*k*v

#Kramer method (see Wikipedia article)
def Kramer(a11, a12, a13, b1,
           a21, a22, a23, b2,
           a31, a32, a33, b3):
    div = Determinant3([[a11, a12, a13], [a21, a22, a23], [a31, a32, a33]])
    x = Determinant3([[b1, a12, a13], [b2, a22, a23], [b3, a32, a33]]) / div
    y = Determinant3([[a11, b1, a13], [a21, b2, a23], [a31, b3, a33]]) / div
    z = Determinant3([[a11, a12, b1], [a21, a22, b2], [a31, a32, b3]]) / div
    return (x, y, z)

# Now, let's calculate e00, e11 and de coefficients from OSL measurements
# These coefficients represent calibration data and should be calculated and stored
# for various frequencies. Coefficients for frequencis in between the calibration
# frequencies can be calculated by linear approximation.
# We are experimenting with one frequency, thus only one set of coefficients for it
# is determined
e00, e11, de = Kramer(1.0, G1 * Gm1, -1.0 * G1, Gm1,
                      1.0, G2 * Gm2, -1.0 * G2, Gm2,
                      1.0, G3 * Gm3, -1.0 * G3, Gm3)
print "Determined (e00, e11, de) calibration coefficients by OSL method:", e00, e11, de
print

#===========================================================================================
# Now, let's verify that these coefficients allow us to recover any actual load impedance
#===========================================================================================

# Suppose we have 200+0j load at the end of coax. Measured impedance Zmeas:
Zmeas = 41.174 - 67.224j
print "Measured Z:", Zmeas
#Calculating reflection coefficient:
Gmeas = Gamma(Zmeas)
print "Gmeas", Gmeas
#Recovering actual reflection coefficient using magic coefficients
G = (Gmeas - e00) / (Gmeas * e11 - de)
print "Corrected G:", G
#Recovering actual impedance from recovered reflection coefficient
print "Recovered Z:", ZFromG(G)
print "Expected: 200.0 + 0j"

print
#Suppoose we have 500-300j load at the end of coax. Doing the same as above:
Zmeas = 12.319 - 75.217j
print "Measured Z:", Zmeas
Gmeas = Gamma(Zmeas)
print "Gmeas", Gmeas
G = (Gmeas - e00) / (Gmeas * e11 - de)
print "Corrected G:", G
print "Recovered Z:", ZFromG(G)
print "Expected: 500.0 - 300j"
